using System;
using System.Linq;
using System.Collections;
using System.Collections.Generic;
using MeshVR;
using SimpleJSON;
using MacGruber;
using MVR.FileManagementSecure;


namespace CheesyFX {
	public class CheesyFX_ExpressionIO : MVRScript {
        public List<DAZMorph> expressionMorphs = new List<DAZMorph>();
        public List<DAZMorph> lHandMorphs = new List<DAZMorph>();
        public List<DAZMorph> rHandMorphs = new List<DAZMorph>();
        public JSONStorableUrl loadExpURL;
        public JSONStorableUrl loadToLeftHandURL;
        public JSONStorableUrl loadToRightHandURL;
        GenerateDAZMorphsControlUI morphControl;
        public const string BASE_DIRECTORY_EXP = "Saves/PluginData/CheesyFX/ExpressionIO/Expressions";
        public const string BASE_DIRECTORY_H = "Saves/PluginData/CheesyFX/ExpressionIO/HandPoses";
        public override void Init(){
            FileManagerSecure.CreateDirectory(BASE_DIRECTORY_EXP);
            FileManagerSecure.CreateDirectory(BASE_DIRECTORY_H);
            DAZCharacterSelector characterSelector = containingAtom.GetStorableByID("geometry") as DAZCharacterSelector;
			morphControl = characterSelector.morphsControlUI;
            List<string> allMorphNames = morphControl.GetMorphDisplayNames();
            foreach(string morphName in allMorphNames){
                DAZMorph morph = morphControl.GetMorphByDisplayName(morphName);
                string region = morph.region.ToLower();
                if(region.Contains("exp")){
                    expressionMorphs.Add(morph);
                }
                if((region.Contains("hand") || region.Contains("finger"))){
                    if(region.Contains("left") || region.Contains("l_") || region.Contains(".l")){
                        lHandMorphs.Add(morph);
                    }
                    else rHandMorphs.Add(morph);
                }
            }
            
            loadExpURL = new JSONStorableUrl("loadExpURL", "", UILoadExprJSON, "jpg", true);
            loadExpURL.hideExtension = true;
            loadExpURL.setCallbackFunction -= UILoadExprJSON;
            loadExpURL.allowFullComputerBrowse = false;
            loadExpURL.allowBrowseAboveSuggestedPath = true;
            loadExpURL.SetFilePath(BASE_DIRECTORY_EXP+"/");
            loadExpURL.setCallbackFunction += UILoadExprJSON;

            loadToLeftHandURL = new JSONStorableUrl("loadLHURL", "", UILoadToLeftHandJSON, "jpg", true);
            loadToLeftHandURL.hideExtension = true;
            loadToLeftHandURL.setCallbackFunction -= UILoadToLeftHandJSON;
            loadToLeftHandURL.allowFullComputerBrowse = false;
            loadToLeftHandURL.allowBrowseAboveSuggestedPath = true;
            loadToLeftHandURL.SetFilePath(BASE_DIRECTORY_H+"/");
            loadToLeftHandURL.setCallbackFunction += UILoadToLeftHandJSON;

            loadToRightHandURL = new JSONStorableUrl("loadRHURL", "", UILoadToRightHandJSON, "jpg", true);
            loadToRightHandURL.hideExtension = true;
            loadToRightHandURL.setCallbackFunction -= UILoadToRightHandJSON;
            loadToRightHandURL.allowFullComputerBrowse = false;
            loadToRightHandURL.allowBrowseAboveSuggestedPath = true;
            loadToRightHandURL.SetFilePath(BASE_DIRECTORY_H+"/");
            loadToRightHandURL.setCallbackFunction += UILoadToRightHandJSON;

            Utils.OnInitUI(CreateUIElement);
            CreateUI();
            
        }
        public JSONClass GetExpressionJSON(){
            JSONClass jc = new JSONClass();
            foreach(DAZMorph morph in expressionMorphs){
                jc[morph.displayName].AsFloat = morph.jsonFloat.val;
            }
            return jc;
        }
        
        public void LoadExpressionJSON(JSONClass jc){
            foreach(DAZMorph morph in expressionMorphs){
                if(jc.HasKey(morph.displayName)) morph.jsonFloat.val = jc[morph.displayName].AsFloat;
            }
        }

        public JSONClass GetLeftHandPoseJSON(){
            JSONClass jc = new JSONClass();
            foreach(DAZMorph morph in lHandMorphs){
                jc[morph.displayName].AsFloat = morph.jsonFloat.val;
            }
            return jc;
        }

        public JSONClass GetRightHandPoseJSON(){
            JSONClass jc = new JSONClass();
            foreach(DAZMorph morph in rHandMorphs){
                jc[morph.displayName].AsFloat = morph.jsonFloat.val;
            }
            return jc;
        }

        public void LoadHandPoseJSON(JSONClass jc, string side){
            string name;
            foreach(string morphName in jc.Keys){
                if(side== "l") name = morphName.Replace("right", "left").Replace("Right", "Left").Replace("R_", "L_").Replace(".R", ".L");
                else name = morphName.Replace("left", "right").Replace("Left", "Right").Replace("L_", "R_").Replace(".L", ".R");
                
                DAZMorph morph = morphControl.GetMorphByDisplayName(name);
                if(morph != null){
                    morph.jsonFloat.val = jc[morphName].AsFloat;
                }
            }
        }

        public void CreateUI(){
            UIDynamicButton button;
            UIDynamicTwinButton tButton;

            button = Utils.SetupButton(this, "Save expression", UISaveExpJSONDialog, false);
            button = Utils.SetupButton(this, "Load expression",  delegate {}, true);
            loadExpURL.RegisterFileBrowseButton(button.button);

            tButton = Utils.SetupTwinButton(this, "Save left hand", UISaveLeftHandJSONDialog, "Save right hand", UISaveRightHandJSONDialog, false);
            tButton = Utils.SetupTwinButton(this, "Load to left hand", delegate {}, "Load to right hand", delegate {}, true);
            loadToLeftHandURL.RegisterFileBrowseButton(tButton.buttonLeft);
            loadToRightHandURL.RegisterFileBrowseButton(tButton.buttonRight);
            
        }

        public void UISaveExpJSONDialog(){
            SuperController sc = SuperController.singleton;
            sc.GetMediaPathDialog(UISaveExpJSON, "jpg", BASE_DIRECTORY_EXP, false, true, false, null, true, null, false, false);
            sc.mediaFileBrowserUI.SetTextEntry(true);
            if(sc.mediaFileBrowserUI.fileEntryField != null){
                string filename = DateTime.Now.ToString("yyyyMMdd_HH-mm-ss");
                sc.mediaFileBrowserUI.fileEntryField.text = filename;
                sc.mediaFileBrowserUI.ActivateFileNameField();
            }
        }

        public void UISaveLeftHandJSONDialog(){
            SuperController sc = SuperController.singleton;
            sc.GetMediaPathDialog(UISaveLeftHandPoseJSON, "jpg", BASE_DIRECTORY_H, false, true, false, null, true, null, false, false);
            sc.mediaFileBrowserUI.SetTextEntry(true);
            if(sc.mediaFileBrowserUI.fileEntryField != null){
                string filename = DateTime.Now.ToString("yyyyMMdd_HH-mm-ss");
                sc.mediaFileBrowserUI.fileEntryField.text = filename;
                sc.mediaFileBrowserUI.ActivateFileNameField();
            }
        }

        public void UISaveRightHandJSONDialog(){
            SuperController sc = SuperController.singleton;
            sc.GetMediaPathDialog(UISaveRightHandPoseHJSON, "jpg", BASE_DIRECTORY_H, false, true, false, null, true, null, false, false);
            sc.mediaFileBrowserUI.SetTextEntry(true);
            if(sc.mediaFileBrowserUI.fileEntryField != null){
                string filename = DateTime.Now.ToString("yyyyMMdd_HH-mm-ss");
                sc.mediaFileBrowserUI.fileEntryField.text = filename;
                sc.mediaFileBrowserUI.ActivateFileNameField();
            }
        }

        public void UISaveExpJSON(string path)
        {
            if (string.IsNullOrEmpty(path))
                return;
            string path1 = path.Replace('\\', '/')+".eIO";
            JSONClass jc = GetExpressionJSON();
            SaveJSON(jc, path1);
            SuperController.singleton.DoSaveScreenshot(path+".jpg");
        }

        public void UISaveLeftHandPoseJSON(string path)
        {
            if (string.IsNullOrEmpty(path))
                return;
            string path1 = path.Replace('\\', '/')+".hIO";
            JSONClass jc = GetLeftHandPoseJSON();
            SaveJSON(jc, path1);
            SuperController.singleton.DoSaveScreenshot(path+".jpg");
        }

        public void UISaveRightHandPoseHJSON(string path)
        {
            if (string.IsNullOrEmpty(path))
                return;
            string path1 = path.Replace('\\', '/')+".hIO";
            JSONClass jc = GetRightHandPoseJSON();
            SaveJSON(jc, path1);
            SuperController.singleton.DoSaveScreenshot(path+".jpg");
        }

        public void UILoadExprJSON(string url){
            JSONClass jc = LoadJSON(url.Replace(".jpg",".eIO")).AsObject;
            if(jc != null) LoadExpressionJSON(jc);
        }

        public void UILoadToLeftHandJSON(string url){
            JSONClass jc = LoadJSON(url.Replace(".jpg",".hIO")).AsObject;
            if(jc != null) LoadHandPoseJSON(jc, "l");
        }
        
        public void UILoadToRightHandJSON(string url){
            JSONClass jc = LoadJSON(url.Replace(".jpg",".hIO")).AsObject;
            if(jc != null) LoadHandPoseJSON(jc, "r");
        }
    }
}